<?php
defined('BASEPATH') OR exit('No direct script access allowed');

class Solicitud extends CI_Controller {

    const AUDIFONO_1 = 901;
    const AUDIFONO_2 = 902;
    const IMPLANTE_1 = 903;
    const IMPLANTE_2 = 904;

    private $allowed_target;


    public function __construct()
    {
        parent::__construct();
        
        if(is_null($this->session->userdata('logged_in')))
            redirect('login');

        $this->allowed_target = array(self::AUDIFONO_1, self::AUDIFONO_2, self::IMPLANTE_1, self::IMPLANTE_2);
        
        $this->section = 'Solicitudes';
    }

	public function index()
	{
        try {

            Permission::check(__METHOD__);

            $this->load->model('hipoacusia_model');
            
            if($this->session->userdata('usuario')->IdRol == getRol('USUARIO'))
            {
                $IdUsuarioCarga = $this->session->userdata('usuario')->IdUsuario;
            }
            elseif($this->session->userdata('usuario')->IdRol == getRol('ADMINISTRADOR'))
            {
                $IdUsuarioCarga = null;
            }
            elseif($this->session->userdata('usuario')->IdRol == getRol('VALIDA'))
            {
                $IdUsuarioCarga = null;
            }
            elseif($this->session->userdata('usuario')->IdRol == getRol('COMPRAS'))
            {
                $IdUsuarioCarga = null;
            }
                
            $data['invitados'] = $this->hipoacusia_model->getInvitadosFromUser($IdUsuarioCarga);
            $data['result'] = $this->hipoacusia_model->get($IdUsuarioCarga);
            $data['content'] = $this->load->view('solicitud/grilla', $data, true);

            add_log('Solicitud - Listado');

            $this->load->view('template', $data);
            
        } catch (Exception $e) {
            go_error($e);
        }
	}

    public function send_email()
    {
        $r = new StdClass;
        $r->status = false;
        $r->msg = null;

        try {

            Permission::check(__METHOD__);

            if(!$this->input->post())
                throw new Exception("Sin datos para enviar el email.");

            $id_hipoacusia = $this->input->post('id_hipoacusia');
            $email_target = $this->input->post('email_target');
            $email_subject = $this->input->post('email_subject');
            $email_content = $this->input->post('email_content');

            // verifica que el id hipoacusia exista
            if(!$id_hipoacusia)
                throw new Exception("Id de hipoacusia nulo.");

            if(!$email_target)
                throw new Exception("Falta(n) el/los destinatarios.");

            if(!$email_subject)
                throw new Exception("Falta completar el asunto del email.");

            if(!$email_content)
                throw new Exception("Falta completar el contenido del email.");

            $this->load->model('hipoacusia_model');
            $row = $this->hipoacusia_model->getById($id_hipoacusia);
            if(!$row)
                throw new Exception("No se encuentra el registro.");

            // busca los emails del target en cuestion
            if(strtoupper($email_target)=='SOPORTE')
            {
                $lista_de_emails = SOPORTE_EMAIL;
            }
            elseif(strtoupper($email_target)=='REFERENTE')
            {
                $lista_de_emails = getEmailByIdUsuario(getFieldValue($row, 'h.IdUsuarioCarga'));  
            }
            else
            {
                $lista_de_emails = getEmailsByRol(strtoupper($email_target));  
            }

            $this->load->model('correo_model');

            $info = array();
            $info['to'] = $lista_de_emails;  
            $info['cco'] = getEmailsByRol('ADMINISTRADOR');
            $info['subject'] = $email_subject;
            $info['body'] = $email_content;


            // insertar el email en Correos para q sea enviado
            if($this->correo_model->insert($info, $id_hipoacusia))
            {
                $r->status = true;
                $r->msg = 'El email será enviado a la brevedad.';
            }
            
        } catch (Exception $e) {
            $r->msg = $e->getMessage();
        }

        add_log('Solicitud - Send Email');

        echo json_encode($r);
    }

    public function add($target = null)
    {
        try {

            Permission::check(__METHOD__);
            
            if(!$target || !in_array($target, $this->allowed_target))
                throw new Exception("Tipo de solicitud indefinida.[".__METHOD__."]");

            $this->load->model('estado_model');
            
            // lo defino para q no rompa la vista
            $data['row'] = null;
            $data['target'] = $target;
            $data['operation'] = 'add';
            $data['referente'] = getReferente($this->session->userdata('usuario')->IdUsuario);

            $form = array();
            $form['datos_paciente'] = $this->load->view('solicitud/datos_paciente', $data, true);
            $form['datos_tutor'] = $this->load->view('solicitud/datos_tutor', $data, true);
            $form['datos_obra_social'] = $this->load->view('solicitud/datos_obra_social', $data, true);
            $form['historia_clinica'] = $this->load->view('solicitud/historia_clinica', $data, true);
            $form['pruebas_general'] = $this->load->view('solicitud/pruebas_general', $data, true);
            $form['datos_evaluacion'] = $this->load->view('solicitud/datos_evaluacion', $data, true);

            // nueva solicitud para audifono/s
            if($target==self::AUDIFONO_1 || $target==self::AUDIFONO_2)
            {
                $form['datos_audifonos'] = $this->load->view('solicitud/datos_audifonos', $data, true);
            }
            // nueva solicitud para implante/s
            elseif($target==self::IMPLANTE_1 || $target==self::IMPLANTE_2)
            {
                $form['datos_implantes'] = $this->load->view('solicitud/datos_implantes', $data, true);
                $form['pruebas_implantes'] = $this->load->view('solicitud/pruebas_implantes', $data, true);
            }

            $form['final'] = $this->load->view('solicitud/final', $data, true);

            // traigo la lista de los correos
            $data['emails'] = null;
            $form['lista_correos'] = $this->load->view('solicitud/lista_correos', $data, true);

            // busca los archivos de esta solicitud en particular
            $data['list_files'] = null;
            $form['lista_archivos'] = $this->load->view('solicitud/lista_files', $data, true);

            $data['title'] = getTipoSolicitud($target);
            $data['form'] = $form;
            $data['content'] = $this->load->view('solicitud/form', $data, true);

            add_log('Solicitud - Add');

            $this->load->view('template', $data);

        } catch (Exception $e) {
            go_error($e);
        }
    }


    public function view($id_hipoacusia = null)
    {
        try {

            Permission::check(__METHOD__);
            
            if(!$id_hipoacusia)
                throw new Exception("Id de hipoacusia nulo.[".__METHOD__."]");

            $this->load->model('hipoacusia_model');
            $row = $this->hipoacusia_model->getById($id_hipoacusia);
            if(!$row)
                throw new Exception("No se encuentra el registro.[".__METHOD__."]");

            $target = getFieldValue($row, 'h.QueSeAutoriza');

            if(!$target || !in_array($target, $this->allowed_target))
                throw new Exception("Tipo de solicitud indefinida.[".__METHOD__."]");
            
            $this->load->model('estado_model');

            $data['row'] = $row;
            $data['target'] = $target;
            $data['operation'] = 'view';
            $data['id_hipoacusia'] = $id_hipoacusia;
            $data['referente'] = getReferente(getFieldValue($row, 'h.IdUsuarioCarga'));

            $form = array();
            $form['datos_paciente'] = $this->load->view('solicitud/datos_paciente', $data, true);
            $form['datos_tutor'] = $this->load->view('solicitud/datos_tutor', $data, true);
            $form['datos_obra_social'] = $this->load->view('solicitud/datos_obra_social', $data, true);
            $form['historia_clinica'] = $this->load->view('solicitud/historia_clinica', $data, true);
            $form['pruebas_general'] = $this->load->view('solicitud/pruebas_general', $data, true);
            $form['datos_evaluacion'] = $this->load->view('solicitud/datos_evaluacion', $data, true);

            // nueva solicitud para audifono/s
            if($target==self::AUDIFONO_1 || $target==self::AUDIFONO_2)
            {
                $form['datos_audifonos'] = $this->load->view('solicitud/datos_audifonos', $data, true);
            }
            // nueva solicitud para implante/s
            elseif($target==self::IMPLANTE_1 || $target==self::IMPLANTE_2)
            {
                $form['datos_implantes'] = $this->load->view('solicitud/datos_implantes', $data, true);
                $form['pruebas_implantes'] = $this->load->view('solicitud/pruebas_implantes', $data, true);
            }

            // traigo la lista de los correos
            $this->load->model('correo_model');
            $data['emails'] = $this->correo_model->getCorreosByIdRelacion($id_hipoacusia);
            $form['lista_correos'] = $this->load->view('solicitud/lista_correos', $data, true);

            // busca los archivos de esta solicitud en particular
            $data['list_files'] = $this->getDocumentacion($id_hipoacusia);
            $form['lista_archivos'] = $this->load->view('solicitud/lista_files', $data, true);

            // inicio bloque invitar
            $this->load->model('invitado_model');
            $data['invitar'] = $this->invitado_model->getInvitadoByIdRelacion($id_hipoacusia);
            $except = array();
            array_push($except, $this->session->userdata('usuario')->IdUsuario);
            if($data['invitar'])
            {
                foreach ($data['invitar'] as $share)
                {
                    array_push($except, $share->IdUsuarioInvitado);
                }
            }
            $data['lista_medicos'] = $this->invitado_model->getListaMedicos($except);
            $form['lista_invitar'] = $this->load->view('solicitud/lista_invitar', $data, true);
            // fin bloque invitar

            $data['title'] = getTipoSolicitud($target);
            $data['form'] = $form;
            $data['content'] = $this->load->view('solicitud/form', $data, true);

            add_log('Solicitud - View');

            $this->load->view('template', $data);

        } catch (Exception $e) {
            go_error($e);
        }
    }

    public function invitacion_add()
    {
        $r = new StdClass;
        $r->status = false;
        $r->msg = null;

        try {

            if(!$this->input->post())
                throw new Exception("Sin información para guardar.[".__METHOD__."]");
            
            Permission::check(__METHOD__);

            $id_hipoacusia = $this->input->post('id_hipoacusia');
            if(!$id_hipoacusia)
                throw new Exception("Id de hipoacusia nulo.[".__METHOD__."]");

            $this->load->model('hipoacusia_model');
            $row = $this->hipoacusia_model->getById($id_hipoacusia);
            if(!$row)
                throw new Exception("No se encuentra el registro.[".__METHOD__."]");

            if(!is_owner($row))
                throw new Exception("Sin permiso para ejecutar esta funcionalidad.[".__METHOD__."]");

            $id_usuario_invitado = $this->input->post('IdUsuarioInvitado');
            if(!$id_usuario_invitado)
                throw new Exception("Debe seleccionar un profesional para invitar.");

            $permiso_escritura = $this->input->post('PermisoEscritura');
            if(strlen($permiso_escritura)<=0)
                throw new Exception("Debe seleccionar si le otorga permiso para editar o no.");

            $id_usuario_titular = $this->session->userdata('usuario')->IdUsuario;

            $post = new StdClass;
            $post->IdUsuarioTitular = $id_usuario_titular;
            $post->IdUsuarioInvitado = $id_usuario_invitado;
            $post->IdAplicativo = ID_APLICATIVO;
            $post->IdRelacion = $id_hipoacusia;
            $post->PermisoEscritura = $permiso_escritura;
            $post->IdUsuarioCarga = $id_usuario_titular;
            $post->FechaCarga = date('Y-m-d');

            $this->db->trans_begin();
            $this->load->model('invitado_model');
            if($this->invitado_model->add($post))
            {
                $this->load->model('usuario_model');
                $user_owner = $this->usuario_model->getById($id_usuario_titular);
                $user_invitado = $this->usuario_model->getById($id_usuario_invitado);
                
                $body = "El referente " . $user_owner->Apellido . ' ' . $user_owner->Nombre . " ha compartido la solicitud ID#{$id_hipoacusia} con " . $user_invitado->Apellido . ' ' . $user_invitado->Nombre . ".";

                // le envia email al invitado con copia al owner
                $options = array();
                $options['to'] = $user_invitado->Email;
                $options['cc'] = $user_owner->Email;
                $options['cco'] = getEmailsByRol('ADMINISTRADOR');
                $options['subject'] = 'Solicitud compartida';
                $options['body']  = $body;
                $options['mostrar']  = false;
                
                if(!notification(null, $id_hipoacusia, $options))
                    throw new Exception("Falla al enviar el email de aviso al invitado.");     

                $this->db->trans_commit();
                
                $r->status = true;
                $r->msg = 'La operación se realizó con éxito.';
            }
            else
            {
                $this->db->trans_rollback();
            }

        } catch (Exception $e) {
            $r->msg = $e->getMessage();
        }

        add_log('Solicitud - Invitation - Add');

        echo json_encode($r);
    }

    public function invitacion_del($id_invitacion = null)
    {
        try {
            
            Permission::check(__METHOD__);

            if(!$id_invitacion)
                throw new Exception("Id de registro nulo.[".__METHOD__."]");

            $this->load->model('invitado_model');
            $invitacion = $this->invitado_model->getById($id_invitacion);
            if(!$invitacion)
                throw new Exception("No se encuentra el registro.[".__METHOD__."]");

            if(!$invitacion->IdAplicativo==ID_APLICATIVO || !(int)$invitacion->IdRelacion)
                throw new Exception("Error en datos para el invitado.[".__METHOD__."]");

            $this->load->model('hipoacusia_model');
            $row = $this->hipoacusia_model->getById($invitacion->IdRelacion);
            if(!$row)
                throw new Exception("No se encuentra el registro.[".__METHOD__."]");

            if(!is_owner($row))
                throw new Exception("Sin permiso para ejecutar esta funcionalidad.[".__METHOD__."]");

            $this->db->trans_begin();
            $this->load->model('invitado_model');
            if($this->invitado_model->del($id_invitacion))
            {
                $id_usuario_invitado = $invitacion->IdUsuarioInvitado;
                if(!$id_usuario_invitado)
                    throw new Exception("No se pudo encontrar el usuario invitado.");

                $id_usuario_titular = $invitacion->IdUsuarioTitular;
                if(!$id_usuario_titular)
                    throw new Exception("No se pudo encontrar el usuario que generó la invitación.");

                $this->load->model('usuario_model');
                $user_owner = $this->usuario_model->getById($id_usuario_titular);
                $user_invitado = $this->usuario_model->getById($id_usuario_invitado);
                $id_hipoacusia = $invitacion->IdRelacion;

                $body = "El referente " . $user_owner->Apellido . ' ' . $user_owner->Nombre . " ha des-compartido la solicitud ID#{$id_hipoacusia} con " . $user_invitado->Apellido . ' ' . $user_invitado->Nombre . ".";

                // le envia email al invitado con copia al owner
                $options = array();
                $options['to'] = $user_invitado->Email;
                $options['cc'] = $user_owner->Email;
                $options['cco'] = getEmailsByRol('ADMINISTRADOR');
                $options['subject'] = 'Solicitud des-compartida';
                $options['body']  = $body;
                $options['mostrar']  = false;
                
                if(!notification(null, $id_hipoacusia, $options))
                    throw new Exception("Falla al enviar el email de aviso al invitado.");     

                $this->db->trans_commit();

                add_log('Solicitud - Invitation - Delete');

                redirect(go_back());
            }
            else
            {
                $this->db->trans_rollback();
                throw new Exception("Error al interntar eliminar invitación.[".__METHOD__."]");
            }
 
        } catch (Exception $e) {            
            go_error($e);
        }
    }

    public function edit($id_hipoacusia = null)
    {
        try {

            Permission::check(__METHOD__);
            
            if(!$id_hipoacusia)
                throw new Exception("Id de hipoacusia nulo.[".__METHOD__."]");

            $this->load->model('hipoacusia_model');
            $row = $this->hipoacusia_model->getById($id_hipoacusia);
            if(!$row)
                throw new Exception("No se encuentra el registro.[".__METHOD__."]");

            // si el estado NO es (los definidos en Permission) no deberia poder subir más documentación
            Permission::check(__METHOD__, getFieldValue($row, 'h.IdEstado'));

            if(is_owner($row) || is_invitado_escritura($row) || is_administrator())
            {
                // puede seguir
            }
            else
            {
                throw new Exception("Sin permiso para la operación.[".__METHOD__."]"); 
            }

            $target = getFieldValue($row, 'h.QueSeAutoriza');
            
            if(!$target || !in_array($target, $this->allowed_target))
                throw new Exception("Tipo de solicitud indefinida.[".__METHOD__."]");

            $this->load->model('estado_model');
            
            $data['row'] = $row;
            $data['target'] = $target;
            $data['operation'] = 'edit';
            $data['id_hipoacusia'] = $id_hipoacusia;
            $data['referente'] = getReferente(getFieldValue($row, 'h.IdUsuarioCarga'));

            $form = array();
            $form['datos_paciente'] = $this->load->view('solicitud/datos_paciente', $data, true);
            $form['datos_tutor'] = $this->load->view('solicitud/datos_tutor', $data, true);
            $form['datos_obra_social'] = $this->load->view('solicitud/datos_obra_social', $data, true);
            $form['historia_clinica'] = $this->load->view('solicitud/historia_clinica', $data, true);
            $form['pruebas_general'] = $this->load->view('solicitud/pruebas_general', $data, true);
            $form['datos_evaluacion'] = $this->load->view('solicitud/datos_evaluacion', $data, true);

            // nueva solicitud para audifono/s
            if($target==self::AUDIFONO_1 || $target==self::AUDIFONO_2)
            {
                $form['datos_audifonos'] = $this->load->view('solicitud/datos_audifonos', $data, true);
            }
            // nueva solicitud para implante/s
            elseif($target==self::IMPLANTE_1 || $target==self::IMPLANTE_2)
            {
                $form['datos_implantes'] = $this->load->view('solicitud/datos_implantes', $data, true);
                $form['pruebas_implantes'] = $this->load->view('solicitud/pruebas_implantes', $data, true);
            }

            $form['final'] = $this->load->view('solicitud/final', $data, true);

            // traigo la lista de los correos
            $this->load->model('correo_model');
            $data['emails'] = $this->correo_model->getCorreosByIdRelacion($id_hipoacusia);
            $form['lista_correos'] = $this->load->view('solicitud/lista_correos', $data, true);

            // busca los archivos de esta solicitud en particular
            $data['list_files'] = $this->getDocumentacion($id_hipoacusia);
            $form['lista_archivos'] = $this->load->view('solicitud/lista_files', $data, true);


            // inicio bloque invitar
            $this->load->model('invitado_model');
            $data['invitar'] = $this->invitado_model->getInvitadoByIdRelacion($id_hipoacusia);
            $except = array();
            array_push($except, $this->session->userdata('usuario')->IdUsuario);
            if($data['invitar'])
            {
                foreach ($data['invitar'] as $share)
                {
                    array_push($except, $share->IdUsuarioInvitado);
                }
            }
            $data['lista_medicos'] = $this->invitado_model->getListaMedicos($except);
            $form['lista_invitar'] = $this->load->view('solicitud/lista_invitar', $data, true);
            // fin bloque invitar


            $data['title'] = getTipoSolicitud($target);
            $data['form'] = $form;
            $data['content'] = $this->load->view('solicitud/form', $data, true);

            add_log('Solicitud - Edit');

            $this->load->view('template', $data);

        } catch (Exception $e) {
            go_error($e);
        }
    }

    public function error($error_code = null, $id = null)
    {
        $data['content'] = $this->load->view('solicitud/error', array('error_code'=>$error_code, 'id'=>$id), true);
        $this->load->view('template', $data);
    }

    public function save()
    {
        try {

            Permission::check(__METHOD__);

            if(!$this->input->post())
                throw new Exception("Sin valores en el post.[".__METHOD__."]");

            $this->load->model('hipoacusia_model');
            $this->load->model('estado_model');

            $post = $this->input->post();

            // add new record
            if($this->input->post('operation')=='add')
            {
                $post['FechaAutorizacion'] = date('Y-m-d');
                $post['IdEstado'] = $this->estado_model->getEstado('INICIADO');
                $post['IdUsuarioCarga'] = $this->session->userdata('usuario')->IdUsuario;

                $id = $this->hipoacusia_model->add($post);
                
                if(!$id)
                    throw new Exception("Imposible insertar registro.[".__METHOD__."]");

                notification('add_solicitud', $id);

                add_log('Solicitud - Save - Add');

                redirect(base_url('index.php/solicitud'));
            }
            // update record
            elseif($this->input->post('operation')=='edit')
            {
                $old = $this->hipoacusia_model->getById($post['id_hipoacusia']);
                if(!$old)
                    throw new Exception("No se encuentra el registro que intenta modificar.[".__METHOD__."]");
                
                $post['FechaAutorizacion'] = date('Y-m-d');
                
                // no lo modifico aca x ahora, solo me quedo con el que se guarda al momento
                // de hacer el add que es el usuario que genera la solicitud.
                //$post['IdUsuarioCarga'] = $this->session->userdata('usuario')->IdUsuario;
                
                $id = $this->hipoacusia_model->edit($post);
                if(!$id)
                    throw new Exception("Imposible editar registro.[".__METHOD__."]");

                $new = $this->hipoacusia_model->getById($post['id_hipoacusia']);
                if(!$new)
                    throw new Exception("Imposible editar registro.(2)[".__METHOD__."]");

                //notification('edit_solicitud', $id, null, array('old'=>$old, 'new'=>$new));  ... por ahora sacar xq sino es mucho mail

                add_log('Solicitud - Save - Edit');

                redirect(base_url('index.php/solicitud'));
            }
            // undefined operation
            else
            {
                throw new Exception("Operación indefinida.[".__METHOD__."]");
            }
            
        } catch (Exception $e) {
            go_error($e);
        }
    }

    private function getDocumentacion($id_hipoacusia)
    {
        if(!$id_hipoacusia)
            throw new Exception("Id de hipoacusia nulo.[".__METHOD__."]");

        $match = $id_hipoacusia . '__';

        // busca los archivos de esta solicitud en particular
        $list_files = array();
        $files = array_diff(scandir(DOC_PATH), array('.', '..'));
        if($files)
        {
            foreach ($files as $file)
            {
                if(strpos($file,$match)===0)
                {
                    array_push($list_files, $file);
                }
            }
        }

        return $list_files;
    }

    public function cursar($id_hipoacusia = null)
    {
        try {

            Permission::check(__METHOD__);
            
            if(!$id_hipoacusia)
                throw new Exception("Id de hipoacusia nulo.[".__METHOD__."]");

            $this->load->model('hipoacusia_model');
            $row = $this->hipoacusia_model->getById($id_hipoacusia);
            if(!$row)
                throw new Exception("No se encuentra el registro.[".__METHOD__."]");

            $this->load->model('estado_model');
            $new_IdEstado = $this->estado_model->getEstado('CURSANDO');
            $old_IdEstado = getFieldValue($row, 'h.IdEstado');
            Permission::change_status($old_IdEstado, $new_IdEstado);
            $id = $this->hipoacusia_model->change_status($id_hipoacusia, $new_IdEstado);
            
            if(!$id)
                throw new Exception("Imposible cambiar estado.[".__METHOD__."]");

            // limpia si o si el campo EstadoDetalle, para que si es un rechazo
            // limpie el detalle del motivo cuando se vuelve a recursar.
            $id = $this->hipoacusia_model->cleanEstadoDetalle($id_hipoacusia);
            if(!$id)
                throw new Exception("Imposible limpiar el detalle del estado.[".__METHOD__."]");

            notification('cursar_solicitud', $id);

            add_log('Solicitud - Cursar');

            go_referer();
            
        } catch (Exception $e) {
            go_error($e);
        }
    }

    public function validar($id_hipoacusia = null)
    {
        try {

            Permission::check(__METHOD__);
            
            if(!$id_hipoacusia)
                throw new Exception("Id de hipoacusia nulo.[".__METHOD__."]");

            $this->load->model('hipoacusia_model');
            $row = $this->hipoacusia_model->getById($id_hipoacusia);
            if(!$row)
                throw new Exception("No se encuentra el registro.[".__METHOD__."]");

            $this->load->model('estado_model');
            $new_IdEstado = $this->estado_model->getEstado('VALIDADO');
            $old_IdEstado = getFieldValue($row, 'h.IdEstado');
            Permission::change_status($old_IdEstado, $new_IdEstado);
            $id = $this->hipoacusia_model->change_status($id_hipoacusia, $new_IdEstado);
            
            if(!$id)
                throw new Exception("Imposible cambiar estado.[".__METHOD__."]");

            notification('validar_solicitud', $id);

            add_log('Solicitud - Validar');

            go_referer();

        } catch (Exception $e) {
            go_error($e);
        }
    }

    public function rechazar()
    {
        try {

            Permission::check(__METHOD__);
            
            if(!$this->input->post())
                throw new Exception("Información del post nula.[".__METHOD__."]");

            $id_hipoacusia = $this->input->post('IdHipoacusia');
            if(!$id_hipoacusia)
                throw new Exception("Id de hipoacusia nulo.[".__METHOD__."]");

            $motivo_rechazo = $this->input->post('motivo_rechazo');
            if(!$motivo_rechazo)
                throw new Exception("Motivo de rechazo nulo.[".__METHOD__."]");

            $this->load->model('hipoacusia_model');
            $row = $this->hipoacusia_model->getById($id_hipoacusia);
            if(!$row)
                throw new Exception("No se encuentra el registro.[".__METHOD__."]");

            $this->load->model('estado_model');
            $new_IdEstado = $this->estado_model->getEstado('RECHAZADO');
            $old_IdEstado = getFieldValue($row, 'h.IdEstado');
            Permission::change_status($old_IdEstado, $new_IdEstado);
            $id = $this->hipoacusia_model->change_status($id_hipoacusia, $new_IdEstado, $motivo_rechazo);
            
            if(!$id)
                throw new Exception("Imposible cambiar estado.[".__METHOD__."]");

            notification('rechazar_solicitud', $id, null, $motivo_rechazo);

            add_log('Solicitud - Rechazar');

            go_referer();

        } catch (Exception $e) {
            go_error($e);
        }
    }

    public function compras($estado = null, $id_hipoacusia = null)
    {
        try {

            Permission::check(__METHOD__);

            if(!$id_hipoacusia)
                throw new Exception("Id de hipoacusia nulo.[".__METHOD__."]");

            if(!$estado)
                throw new Exception("Estado de compra nulo.[".__METHOD__."]");

            $estado = trim($estado);

            $this->load->model('hipoacusia_model');
            $row = $this->hipoacusia_model->getById($id_hipoacusia);
            if(!$row)
                throw new Exception("No se encuentra el registro.[".__METHOD__."]");

            $this->load->model('estado_model');
            $new_IdEstado = $this->estado_model->getEstado(strtoupper($estado));
            $old_IdEstado = getFieldValue($row, 'h.IdEstado');
            Permission::change_status($old_IdEstado, $new_IdEstado);
            
            $this->db->trans_begin();
            $id = $this->hipoacusia_model->change_status($id_hipoacusia, $new_IdEstado);
            
            if(!$id)
                throw new Exception("Imposible cambiar estado.[".__METHOD__."]");

            //if($new_IdEstado == $this->estado_model->getEstado('ENTREGADO'))
            if($new_IdEstado == $this->estado_model->getEstado('PROCESADO'))
                $this->update_stock($id);

            $this->db->trans_commit();

            notification("{$estado}_solicitud", $id);

            add_log('Solicitud - Estado - ' . $estado);

            go_referer();

        } catch (Exception $e) {
            $this->db->trans_rollback();
            go_error($e);
        }
    }

    private function update_stock($id_hipoacusia)
    {
        try {

            if(!$id_hipoacusia)
                throw new Exception("Id de hipoacusia nulo.[".__METHOD__."]");

            $this->load->model('hipoacusia_model');
            $row = $this->hipoacusia_model->getById($id_hipoacusia);
            if(!$row)
                throw new Exception("No se encuentra el registro.[".__METHOD__."]");

            /*
            * Solo si el estado es ENTREGADO debe actualizar el stock
            * del producto más viejo q tenga stock disponible y continuar
            * con el sigte si no le alcanza la cantidad. 
            * Dependiendo de QueSeAutoriza (audifono/implante) debe buscar
            * actualizar el producto de dicho tipo. 
            */

            $this->load->model('estado_model');
            //if(getFieldValue($row, 'h.IdEstado') != $this->estado_model->getEstado('ENTREGADO'))
            //if(getFieldValue($row, 'h.IdEstado') != $this->estado_model->getEstado('PROCESADO'))
            //    throw new Exception("El estado actual no permite actualizar el stock.[".__METHOD__."]");

            $this->load->model('producto_model');

            // 0) detecto el tipo de producto
            $tipo = getTipoProducto(getFieldValue($row, 'h.QueSeAutoriza'));
            if(!$tipo)
                throw new Exception("No se pudo obtener el tipo de producto.[".__METHOD__."]");

            // 1) busco los productos del tipo, sino hay throw
            $productos = $this->producto_model->getProductosPorTipo($tipo['tipo']);
            if(!$productos)
                throw new Exception("No se encontraron productos para el stock.[".__METHOD__."]");

            // 2) totalizo para ver si me alcanza, sino throw
            $total_disponible = 0;
            foreach ($productos as $prod)
            {
                $total_disponible += $prod->Stock;
            }

            if($total_disponible < $tipo['cantidad'])
                throw new Exception("El producto no cuenta con stock suficiente.[".__METHOD__."]");
      

            // 3) recorro x el más viejo y empiezo a restarle
            // sino me alcanza con este registro paso al sigte.
            // asi hasta completar el total a restar del stock
            foreach ($productos as $prod)
            {
                if($prod->Stock >= $tipo['cantidad'])
                {
                    $this->producto_model->resto_stock($prod->IdProducto, $tipo['cantidad']);
                    break;
                }
                else
                {
                    $this->producto_model->resto_stock($prod->IdProducto, $prod->Stock);
                    $tipo['cantidad'] -= $prod->Stock;
                }
            }
            

        } catch (Exception $e) {
            go_error($e);
        }
    }

    public function replicar($target, $id_hipoacusia)
    {
        try {

            Permission::check(__METHOD__);

            if(!$target || !in_array($target, $this->allowed_target))
                throw new Exception("Tipo de solicitud indefinida.[".__METHOD__."]");

            if(!$id_hipoacusia)
                throw new Exception("Id de hipoacusia nulo.[".__METHOD__."]");

            $this->load->model('hipoacusia_model');
            $row = $this->hipoacusia_model->getById($id_hipoacusia);
            if(!$row)
                throw new Exception("No se encuentra el registro.[".__METHOD__."]");

            $id = $this->hipoacusia_model->replicar($id_hipoacusia, $target, $row);
            
            if(!$id)
                throw new Exception("Imposible replicar registro.[".__METHOD__."]");

            // replico los invitados??? NO!
            // replico el historial de mails?? NO
            
            // copio todos los docs del origen al destino
            $files = $this->getDocumentacion($id_hipoacusia);
            //[0] => 4__1538294102__nombre-del-archivo.txt
            
            if($files)
            {
                foreach ($files as $v)
                {
                    $filename = end(explode('__',$v));
                    $new_filename = $id.'__'.time().'__'.$filename;
                    if(!copy(DOC_PATH.$v, DOC_PATH.$new_filename))
                        throw new Exception("Error al intentar copiar documentación.[".__METHOD__."]");
                        
                }  
            }

            notification("replicar_solicitud", $id);

            add_log('Solicitud - Replicar');

            redirect(base_url('index.php/solicitud'));

        } catch (Exception $e) {
            go_error($e);
        }
    }

}
